<?php 
function file_rdf_datetime($timestamp = NULL, $tz_offset = NULL, $tz_name = NULL, $datatype = 'xsd:dateTime') {
  
  if (module_exists('rdf')) {
    return file_rdf_datetime($timestamp, $tz_offset, $tz_name, $datatype);
  }

  if (is_numeric($timestamp)) {
    $timestamp = (int)$timestamp;
  }
  else if (is_string($timestamp)) {
    $timestamp = strtotime($timestamp);
  }
  else if (is_null($timestamp)) {
    $timestamp = gmmktime();
  }

  $datetime = format_date($timestamp, 'custom', 'Y-m-d\TH:i:sO', (int)$tz_offset);
  $datetime = str_replace('+0000', 'Z', $datetime); // clean up format_date() result
  $datetime = preg_replace('/([+-])(\d{2}):?(\d{2})$/', '\1\2:\3', $datetime);

  // This is a non-standard extension for outputting the time zone name
  // appended to the datetime literal value. If you don't know why you'd do
  // this, then you don't need this. However, if you *do* need this, make
  // sure to also specify another literal datatype than 'xsd:dateTime' in
  // the $datatype argument in order to avoid overloading the XSD semantics.
  // Example output: "2009-08-15T23:59:59-04:00 America/New_York"^^$datatype
  if ($tz_name) {
    $datetime .= ' ' . (string)$tz_name;
  }

  return (object) array('value' => $datetime, 'language' => NULL, 'datatype' => $datatype);
}
/**
 * Determines whether a given statement exists.
 *
 * @param $subject
 * @param $predicate
 * @param $object
 * @param $options
 * @return
 *   TRUE if the statement exists, FALSE otherwise.
 */
function file_rdf_exists($subject, $predicate = NULL, $object = NULL, array $options = array()) {
  return file_rdf_count($subject, $predicate, $object, $options) > 0;
}

/**
 * Returns the number of matching statements.
 *
 * @param $subject
 * @param $predicate
 * @param $object
 * @param $options
 * @return
 */
function file_rdf_count($subject = NULL, $predicate = NULL, $object = NULL, array $options = array()) {
  $results = file_rdf_query($subject, $predicate, $object, $options);
  return count(is_object($results) ? $results->to_array() : $results);
}

/**
 * Inserts multiple new statements.
 *
 * @param $statements
 * @return
 *   TRUE if all statements were successfully inserted, FALSE otherwise.
 */
function file_rdf_insert_all($statements, array $options = array()) {
  
  if (module_exists('rdf')) {
    return rdf_insert_all($statements, $options);
  }

  $result = TRUE;
  foreach ($statements as $statement) {
    $result = call_user_func_array('file_rdf_insert', array_merge($statement, array($options))) && $result;
  }
  return $result;
}
/**
 * Deletes an existing statement.
 *
 * @param $subject
 * @param $predicate
 * @param $object
 * @param $options
 * @return
 *   TRUE, or a repository-specific non-NULL value, if the statement was
 *   successfully deleted; FALSE if an error occurred.
 */
function file_rdf_delete($subject, $predicate, $object, array $options = array()) {

  if (module_exists('rdf')) {
    return rdf_delete($subject, $predicate, $object, $options);
  }
  
  global $base_url;

  $s = isset($subject) ? '<'. $subject .'>' : '?s';
  $p = isset($predicate) ? '<'. $predicate .'>' : '?p';
  $o = isset($object) ?  $object : '?o';
  
  if(is_object($o))
    $o = $o->value;
  
  if($o != '?o')
    $data = is_numeric($o) ? (int) $o : '"'. $o . '"';
  
  if(empty($data))
    $data = $o;
  
  $store = file_rdf_getStore();
  $res = $store->query('DELETE {'. $s .' '. $p .' '. $data .'}');
   
  if($err = $store->getErrors()){
    watchdog('file_rdf_delete', 'error delete source %source', array('%source' => $s), WATCHDOG_ERROR);
    return FALSE;
  }
  return TRUE;
}
function _file_rdf_query_arg($uri_or_qname) {
  if (is_string($uri_or_qname) && preg_match('/^[\w]+:[\w]+$/i', $uri_or_qname))
    return file_rdf_qname_to_uri($uri_or_qname);
  return is_object($uri_or_qname) ? (string)$uri_or_qname : $uri_or_qname;
}
/**
 * Inserts a new statement.
 *
 * @param $subject
 * @param $predicate
 * @param $object
 * @param $options
 * @return
 *   TRUE, or a repository-specific non-NULL value, if the statement was
 *   successfully inserted; FALSE if an error occurred.
 */
function file_rdf_insert($subject, $predicate, $object, array $options = array()) {

  if (module_exists('rdf')) {
    return rdf_insert($subject, $predicate, $object, $options);
  }
  
  global $base_url;
  $subject   = $subject   ? _file_rdf_query_arg($subject)   : $subject;
  $predicate = $predicate ? _file_rdf_query_arg($predicate) : $predicate;
  $repos     = isset($options['repository']) ? $options['repository'] : NULL;
  
  $s = isset($subject) ? '<'. $subject .'>' : '?s';
  $p = isset($predicate) ? '<'. $predicate .'>' : '?p';
  $o = isset($object) ?  $object  : '?o';
  
  $data = 0;
  if(is_object($o))
    $o = $o->value;
  if($o != '?o')
    !is_string($o) ? $data = $o : $data = '"'. $o . '"';
  
  

  $store = file_rdf_getStore();
  $res = $store->query('INSERT INTO <'. $base_url .'/'. $options['repository'] .'> {'. $s .' '. $p .' '. $data .' }');
  
  if($err = $store->getErrors()){
    watchdog('file_rdf_insert', 'error insert source %source', array('%source' => $s), WATCHDOG_ERROR);
    return FALSE;
  }
  return TRUE;
}
function file_rdf_denormalize($input) {
  
  if (module_exists('rdf')) {
    return rdf_denormalize($input);
  }

  $output = array();
  foreach ($input as $subject => $predicates) {
    foreach ($predicates as $predicate => $objects) {
      if (!is_array($objects)) {
        $output[] = array($subject, $predicate, $objects);
      }
      else {
        foreach ($objects as $object) {
          $output[] = array($subject, $predicate, $object);
        }
      }
    }
  }
  return $output;
}
/**
 * Implementation of hook_file_rdf_namespaces().
 */
function file_rdf_rdf_namespaces() {

  $namespaces = array();
  if (!module_exists('rdf')) {
    $namespaces = array(
      '_'        => 'http://bnode.net/',
      'rdf'      => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
      'rdfs'     => 'http://www.w3.org/2000/01/rdf-schema#',
      'xsi'      => 'http://www.w3.org/2001/XMLSchema-instance#',
      'xsd'      => 'http://www.w3.org/2001/XMLSchema#',
      'owl'      => 'http://www.w3.org/2002/07/owl#',
      'dc'       => 'http://purl.org/dc/elements/1.1/',
      'dcterms'  => 'http://purl.org/dc/terms/',
      'dcmitype' => 'http://purl.org/dc/dcmitype/',
      'foaf'     => 'http://xmlns.com/foaf/0.1/',
     // 'iptc'     => 'http://iptc.org/std/Iptc4xmpCore/1.0/xmlns/',
    );
  }
  return $namespaces;
}
function file_rdf_qname_to_uri($qname) {
 
  if (module_exists('rdf')) {
    return rdf_qname_to_uri($qname);
  }
  
  if (file_rdf_is_valid_qname($qname)) {
    $namespaces = file_rdf_get_namespaces();
    list($prefix, $local_part) = explode(':', $qname, 2);
    if (isset($namespaces[$prefix])) {
      return $namespaces[$prefix] . $local_part;
    }
  }
  
  return $qname; // FIXME!
}
function file_rdf_is_valid_qname($qname) {
  return preg_match('/^[\w\-]+:[\w\-]+$/', (string)$qname); // FIXME
}
function file_rdf_normalize($input) {
  if (module_exists('rdf')) {
    return rdf_normalize($input);
  } 
  $output = array();
  foreach ($input as $triple) {
    list($subject, $predicate, $object) = $triple;
    $output[(string)$subject][(string)$predicate][] = $object;
  }
  return $output;

}
function file_rdf_get_namespaces($op = NULL) {
  
  if (module_exists('rdf')) {
    return rdf_get_namespaces($op);
  }
  
  global $_file_rdf_namespaces;
  if (empty($_file_rdf_namespaces)) {
    $_file_rdf_namespaces = module_invoke_all('rdf_namespaces');
    ksort($_file_rdf_namespaces);
  }

  switch ($op) {
    case 'rdfa': // only RDFa-enabled namespaces
      $defaults = array('rdf', 'dc', 'foaf');
      $defaults = array_combine($defaults, $defaults);
      return array_intersect_key($_file_rdf_namespaces, array_filter(variable_get('file_rdf_rdfa_prefixes', $defaults), 'is_string'));
    default:     // all namespaces
      return $_file_rdf_namespaces;
  }
}

function file_rdf_flatResult($query) {

  $metadata = array();
  $namespaces = file_rdf_get_namespaces();
  foreach ($query['result'] as $subject => $predicates) {
    foreach ($predicates as $predicate => $objects) {       
      foreach ($objects as $object) {
        $rdf = file_rdf_uri_to_qname($predicate, $namespaces, FALSE);
        $metadata[$rdf] =  $object['value'] ? $object['value'] : $object;
      }
    }
  }
  return $metadata;

}
function file_rdf_uri_to_qname($uri, $gensym = TRUE, $fail = NULL) {
  
  if (module_exists('rdf')) {
    return rdf_uri_to_qname($uri, $gensym, $fail);
  }
  
  global $_file_rdf_namespaces;
  if (empty($_file_rdf_namespaces)) {
    file_rdf_get_namespaces();
  }

  if (!file_rdf_is_valid_uri($uri = (string)$uri)) {
    return $uri;
  }

  $best_prefix = $best_match = '';

  // Attempt to find the longest substring match
  foreach ($_file_rdf_namespaces as $prefix => $match) {
    if (strpos($uri, $match) === 0 && strlen($match) > strlen($best_match)) {
      $best_match = $match;
      $best_prefix = $prefix;
    }
  }  

  // If successful, life is easy
  if (!empty($best_prefix)) {
    $local_part = substr($uri, strlen($best_match));
    return implode(':', array($best_prefix, $local_part));
  }

  // No existing namespace prefix found, make one up
  if ($gensym && preg_match('@([\w\d-_]+)$@', $uri, $matches)) {
    static $gensym = 0;
    $prefix = 'g.' . ++$gensym; // Good ol' Lisp tradition continues...
    $local_part = $matches[1];
    $_file_rdf_namespaces[$prefix] = substr($uri, 0, -strlen($local_part));
    return implode(':', array($prefix, $local_part));
  }

  //trigger_error('Could not convert URI ' . $uri . ' to QName', E_USER_WARNING);
  return $fail;
}
function file_rdf_to_rdf($rows = array(), $s, $p, $o) {

  foreach ($rows['rows'] as $row) {
    $row['s'] = !isset($row['s']) ? $s : $row['s'];
    $row['s type'] = !isset($row['s type']) ? 'uri' : $row['s type'];
    $row['o'] = !isset($row['o']) ? $o : $row['o'];
    $row['o type'] = !isset($row['o type']) ? 'uri' : $row['o type'];
    $row['p'] = !isset($row['p']) ? $p : $row['p'];
    $row['p type'] = !isset($row['p type']) ? 'uri' : $row['p type'];
    $rdf_array[] = _file_rdf_deconstruct_arc2_triple($row);
  }
  return $rdf_array;

}
function file_rdf_value($subject, $predicate, $default = NULL, array $options = array()) {
  if (module_exists('rdf')) {
    return rdf_value($subject, $predicate, $default, $options);
  }
  
  foreach (file_rdf_query($subject, $predicate, NULL, $options) as $statement) {
    return $statement[2]; // object
  }
  return $default;
}
function file_rdf_query($subject = NULL, $predicate = NULL, $object = NULL, array $options = array()) {

  if (module_exists('rdf')) {
    return rdf_query($subject, $predicate, $object, $options);
  }
  
  
  $s = isset($subject) ? '<'. $subject .'>' : '?s';
  $p = isset($predicate) ? '<'. $predicate .'>' : '?p';
  $o = isset($object) ?  $object : '?o';
   
  if($o != '?o') {
    $data = is_numeric($o) ? (int) $o : '"'. $o . '"';
  }else{
    $data = $o;
  }

  
  $store = file_rdf_getStore();
  
  if(isset($subject) && isset($predicate) && isset($object)) {
    $res = $store->query('ASK WHERE { '. $s .' '. $p .' '. $data .' }');
  }else{
    $res = $store->query('SELECT * WHERE { '. $s .' '. $p .' '. $data .' }');
  }
  
  if (!empty($res['result']['rows'])) {
    $res = file_rdf_to_rdf($res['result'], $subject, $predicate, $object);    
  }
  else {
    $res = array();
  }
  
  return $res;
}
//////////////////////////////////////////////////////////////////////////////
// ARC2 interoperability
function _file_rdf_deconstruct_arc2_triple($triple) {

  $subject   = _file_rdf_deconstruct_arc2_value($triple, 's');
  $predicate = _file_rdf_deconstruct_arc2_value($triple, 'p');
  $object    = _file_rdf_deconstruct_arc2_value($triple, 'o');
  return array($subject, $predicate, $object);
}

function _file_rdf_deconstruct_arc2_value($triple, $name) {

  switch ($name == 'p' ? 'iri' : $triple[$name . ' type']) {
    case 'uri':
    case 'iri':
      return $triple[$name];
    case 'bnode':
      return $triple[$name];
    case 'literal':
    case 'literal1':
    case 'literal2':
    case 'literal_long1':
    case 'literal_long2':
      if (isset($triple[$name . ' datatype'])) {
        return (object) array('value' => $triple[$name], 
                              'language' => $triple[$name . ' lang'], 
                              'datatype' => $triple[$name . ' datatype']);
      }
      else {
        return $triple[$name];
      }    case 'var':
      return rdf_var($triple[$name]);
    case 'var':
      return $triple[$name];
  }
}